==> OpenCL-0.8.3.tgz: OpenCL-0.8.3/src/fused.cpp <==
/*************************************************
* BigInt Fused Arithmetic Operations Source File *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/fused.h>
#include <opencl/mp_core.h>

namespace OpenCL {

/*************************************************
* Fused Multiply/Add Operation                   *
*************************************************/
BigInt mul_add(const BigInt& a, const BigInt& b, const BigInt& c)
   {
   if(c.is_negative())
      throw Invalid_Argument("mul_add: Third argument must be positive");

   Sign sign = Positive;
   if(a.sign() != b.sign())
      sign = Negative;

   u32bit a_words = a.sig_words();
   u32bit b_words = b.sig_words();
   u32bit c_words = c.sig_words();

   BigInt z(sign, std::max(a_words + b_words + 1, c_words + 1));
   bigint_mul3(z.get_reg(), a.data(), a_words, b.data(), b_words);

   u32bit carry = 0;
   u32bit z_words = std::max(z.sig_words(), c_words);
   for(u32bit j = 0; j != c_words; j++)
      {
      u64bit sum = ((u64bit)z[j] + c.at(j)) + carry;
      z[j] = (sum & 0xFFFFFFFF);
      carry = (sum >> 32) & 0xFFFFFFFF;
      }
   for(u32bit j = c_words; j != z_words; j++)
      {
      u64bit sum = (u64bit)z[j] + carry;
      z[j] = (sum & 0xFFFFFFFF);
      carry = (sum >> 32) & 0xFFFFFFFF;
      }
   z[z_words] = carry;

   return z;
   }

/*************************************************
* Fused Subtract/Multiply Operation              *
*************************************************/
BigInt sub_mul(const BigInt& a, const BigInt& b, const BigInt& c)
   {
   return (a - b) * c;
   }

}

==> OpenCL-0.8.5.tgz: OpenCL-0.8.5/src/fused.cpp <==
/*************************************************
* BigInt Fused Arithmetic Operations Source File *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/fused.h>
#include <opencl/mp_core.h>

namespace OpenCL {

/*************************************************
* Fused Multiply/Add Operation                   *
*************************************************/
BigInt mul_add(const BigInt& a, const BigInt& b, const BigInt& c)
   {
   if(c.is_negative())
      throw Invalid_Argument("mul_add: Third argument must be positive");

   Sign sign = Positive;
   if(a.sign() != b.sign())
      sign = Negative;

   u32bit a_words = a.sig_words();
   u32bit b_words = b.sig_words();
   u32bit c_words = c.sig_words();

   BigInt z(sign, std::max(a_words + b_words + 1, c_words + 1));
   bigint_mul3(z.get_reg(), a.data(), a_words, b.data(), b_words);

   word carry = 0;
   u32bit z_words = std::max(z.sig_words(), c_words);
   for(u32bit j = 0; j != c_words; j++)
      {
      dword sum = ((dword)z[j] + c.at(j)) + carry;
      z[j] = (sum & MP_WORD_MAX);
      carry = (sum >> MP_WORD_BITS) & MP_WORD_MAX;
      }
   for(u32bit j = c_words; j != z_words; j++)
      {
      dword sum = (dword)z[j] + carry;
      z[j] = (sum & MP_WORD_MAX);
      carry = (sum >> MP_WORD_BITS) & MP_WORD_MAX;
      }
   z[z_words] = carry;

   return z;
   }

/*************************************************
* Fused Subtract/Multiply Operation              *
*************************************************/
BigInt sub_mul(const BigInt& a, const BigInt& b, const BigInt& c)
   {
   return (a - b) * c;
   }

}
